/*
  TOPVIEW.C

  Code fragments for detecting the presence of a multitasker and
  making .C programs "multitasker aware".

*/

#include <dos.h>

#define	FALSE	0
#define TRUE	!FALSE
#define Z_FLAG	0x40


/*  Comment out this DEBUG declaration if you want to use this code in
	a program of your own - it allows a test main() to be compiled when
	defined.
*/
#define DEBUG	1

/* Register structures used by int86() and int86x().
*/
union REGS rg;
struct SREGS srg;


/*

  This will hold the value of the base address of the display.
  It is set to the color text buffer physical address by default.
  You should determine if a mono system is in use and, if so,
  change the value of screen_segm to 0B000h prior to calling
  detect_TopView(). detect_TopView() will return a boolean
  FALSE if a multitasker IS FOUND - this allows it's return value
  to be assigned directly to Borland's DirectVideo variable.

*/
unsigned screen_segm = 0xB800, screen_offs = 0;


/*

  The values in these variables are moved into the AH register prior
  to calling int 0x16 to get keyboard input. detect_TopView() will move
  the value of keyboard_read_fn into keyboard_check_fn if a multitasker
  is found. If you want to support the enhanced keyboard, you should
  determine if one exists and, if so, add 0x10 to each of these variables
  prior to calling detect_TopView().

  If you want to keep running in background when no keyboard input is
  available (to spool output to the printer, etc.) then comment out
  the lines in detect_TopView() that reference these variables.

*/

char keyboard_check_fn	= 1, keyboard_read_fn = 0;


/*
  Determines the presence of a TopView compatible multitasker such as
  OMNIVIEW or DESQview and alters the variables above if one is found.

  Parms:	void
  Result:	Returns FALSE (0) if multitasker IS FOUND, else TRUE (1)

  Note:
		Those using Borland's conio routines can use
		this function to select BIOS output if a multitasker
		is installed by:

			directvideo = detect_TopView();

*/
int detect_TopView( void )
{
	srg.es = screen_segm;
	rg.x.di = screen_offs;
	int86x( 0x10, &rg, &rg, &srg );
	if( srg.es == screen_segm ) return( TRUE );
	screen_segm = srg.es;
	screen_offs = rg.x.di;
	keyboard_check_fn = keyboard_read_fn;
	return( FALSE );
}


/*
  Writes a string to virtual or physical video display memory. DOES NOT
  account for snowy CGAs.

  Parms:	attrib = attribute to use when displaying string
	 		xpos = char row [0-n]
 			ypos = char column [0-n]
 			*string = address of string to write
  Result:	void

*/
void write_string(char *string, char attrib, char xpos, char ypos)
{
  char far *tmp;
  unsigned offset;

	tmp = MK_FP( 0, 0x44A );	/* compute display offset */
	offset = *tmp;			/* of (xpos, ypos) */
	offset *= ypos;
	offset += xpos;
	offset <<= 1;

	tmp = MK_FP( screen_segm, screen_offs );
	tmp += offset;			/* point tmp to display address */

	while( *string ) {
		*tmp++ = *string++;	/* write char/attrib pairs */
		*tmp++ = attrib;
	}
}


/*
  Gets keyboard input if available. Gives up time slice and waits if running
  under a multitasker and no input available.

  Parms:	xscan = unsigned to hold return char/scan code pair
  Result:	Returns FALSE (0) if no input available (and no multitasker
  			present) else returns TRUE (1) and xscan holds extended scan 
			code pair where	LO(xscan) == character_typed.
*/
int  get_kbd_inp( unsigned *xscan )
{
	if( keyboard_check_fn & 1 ) {		/* is "TopView" here? */
		rg.h.ah = keyboard_check_fn;	/* no, check for input */
		int86( 0x16, &rg, &rg );
		if( (rg.x.flags & Z_FLAG) )
			return( FALSE );			/* return FALSE if none */
	}
	rg.h.ah = keyboard_read_fn;			/* read input */
	int86( 0x16, &rg, &rg );
	*xscan = rg.x.ax;					/* record input */
	return( TRUE );						/* return TRUE */
}

#if defined(DEBUG)

#define LO(x)	((x) & 0xFF)
#define HI(x)	((x) >> 8)

/* 	As a test, this reads the first 100 chars from the keyboard (or until
	the ESC key is pressed) and displays them as five rows of red on black
	characters on the CRT. It assumes a Color system and, if no multitasker 
	is present, may cause snow on a CGA.

*/
unsigned c;
char string[2] = {0,0};


main()
{
	int count = 0;

	detect_TopView();
	while( count < 100 ) {

		if( get_kbd_inp( &c ) == FALSE ) /* get keyboard input */
			continue;
		 if( c == 0x011B ) 				 /* look for escape key */
			break;
		 else
			string[0] = LO(c);
            							/* write char */
		 write_string( string, 4, (count % 20), (count / 20) );
		 ++count;             			/* bump count */
	}
}

#endif /* DEBUG (test main) */
